@@ -24,6 +24,7 @@
 #include <KIO/DndPopupMenuPlugin>
 #include <KIO/FileUndoManager>
 #include <KJobWidgets>
+#include <KJobWindows>
 #include <KLocalizedString>
 #include <KPluginFactory>
 #include <KPluginMetaData>
@@ -42,6 +43,7 @@
 #include <QMimeData>
 #include <QProcess>
 #include <QTimer>
+#include <QWindow>
 
 using namespace KIO;
 
@@ -143,6 +145,8 @@ public:
     void addPluginActions(KIO::DropMenu *popup, const KFileItemListProperties &itemProps);
     void doCopyToDirectory();
 
+    QWindow *transientParent();
+
     QPointer<const QMimeData> m_mimeData;
     const QList<QUrl> m_urls;
     QMap<QString, QString> m_metaData;
@@ -301,12 +305,17 @@ void DropJobPrivate::fillPopupMenu(KIO::DropMenu *popup)
     for (const QUrl &url : m_urls) {
         fileItems.append(KFileItem(url));
     }
+    const bool allSourcesAreHttpUrls = std::ranges::all_of(m_urls, [](const auto &url) {
+        return url.scheme().startsWith(QStringLiteral("http"), Qt::CaseInsensitive);
+    });
     const KFileItemListProperties itemProps(fileItems);
 
     Q_EMIT q->popupMenuAboutToShow(itemProps);
 
     const bool sReading = itemProps.supportsReading();
-    const bool sDeleting = itemProps.supportsDeleting();
+    // For http URLs, even though technically the protocol supports deleting,
+    // this never makes sense for a drag operation.
+    const bool sDeleting = allSourcesAreHttpUrls ? false : itemProps.supportsDeleting();
     const bool sMoving = itemProps.supportsMoving();
 
     const int separatorLength = QCoreApplication::translate("QShortcut", "+").size();
@@ -317,8 +326,11 @@ void DropJobPrivate::fillPopupMenu(KIO::DropMenu *popup)
     popupMoveAction->setData(QVariant::fromValue(Qt::MoveAction));
     seq = QKeySequence(Qt::ControlModifier).toString(QKeySequence::NativeText);
     seq.chop(separatorLength);
-    QAction *popupCopyAction = new QAction(i18n("&Copy Here") + QLatin1Char('\t') + seq, popup);
-    popupCopyAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-copy")));
+
+    const QString copyActionName = allSourcesAreHttpUrls ? i18nc("@action:inmenu Download contents of URL here", "&Download Here") : i18n("&Copy Here");
+    const QIcon copyActionIcon = QIcon::fromTheme(allSourcesAreHttpUrls ? QStringLiteral("download") : QStringLiteral("edit-copy"));
+    QAction *popupCopyAction = new QAction(copyActionName + QLatin1Char('\t') + seq, popup);
+    popupCopyAction->setIcon(copyActionIcon);
     popupCopyAction->setData(QVariant::fromValue(Qt::CopyAction));
     seq = QKeySequence(Qt::ControlModifier | Qt::ShiftModifier).toString(QKeySequence::NativeText);
     seq.chop(separatorLength);
@@ -381,6 +393,11 @@ void DropJob::showMenu(const QPoint &p, QAction *atAction)
     }
 
     for (KIO::DropMenu *menu : std::as_const(d->m_menus)) {
+        if (QWindow *transientParent = d->transientParent()) {
+            if (menu->winId()) {
+                menu->windowHandle()->setTransientParent(transientParent);
+            }
+        }
         menu->popup(p, atAction);
     }
 }
@@ -435,6 +452,23 @@ void DropJobPrivate::handleCopyToDirectory()
     bool containsTrashRoot = false;
     for (const QUrl &url : m_urls) {
         const bool local = url.isLocalFile();
+#ifdef Q_OS_LINUX
+        // Check if the file is already in the xdg trash folder, BUG:497390
+        const QString xdgtrash = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/Trash");
+        if (!local /*optimization*/ && url.scheme() == QLatin1String("trash")) {
+            if (url.path().isEmpty() || url.path() == QLatin1String("/")) {
+                containsTrashRoot = true;
+            }
+        } else if (local || url.scheme() == QLatin1String("file")) {
+            if (!url.toLocalFile().startsWith(xdgtrash)) {
+                allItemsAreFromTrash = false;
+            } else if (url.path().isEmpty() || url.path() == QLatin1String("/")) {
+                containsTrashRoot = true;
+            }
+        } else {
+            allItemsAreFromTrash = false;
+        }
+#else
         if (!local /*optimization*/ && url.scheme() == QLatin1String("trash")) {
             if (url.path().isEmpty() || url.path() == QLatin1String("/")) {
                 containsTrashRoot = true;
@@ -442,6 +476,7 @@ void DropJobPrivate::handleCopyToDirectory()
         } else {
             allItemsAreFromTrash = false;
         }
+#endif
         if (url.matches(m_destUrl, QUrl::StripTrailingSlash)) {
             slotDropActionDetermined(KIO::ERR_DROP_ON_ITSELF);
             return;
@@ -503,6 +538,23 @@ void DropJobPrivate::handleCopyToDirectory()
     slotDropActionDetermined(err);
 }
 
+QWindow *DropJobPrivate::transientParent()
+{
+    Q_Q(DropJob);
+
+    if (QWidget *widget = KJobWidgets::window(q)) {
+        QWidget *window = widget->window();
+        Q_ASSERT(window);
+        return window->windowHandle();
+    }
+
+    if (QWindow *window = KJobWindows::window(q)) {
+        return window;
+    }
+
+    return nullptr;
+}
+
 void DropJobPrivate::slotDropActionDetermined(int error)
 {
     Q_Q(DropJob);
@@ -514,8 +566,7 @@ void DropJobPrivate::slotDropActionDetermined(int error)
 
     // There was an error, handle it
     if (error == KIO::ERR_UNKNOWN) {
-        auto *window = KJobWidgets::window(q);
-        KIO::DropMenu *menu = new KIO::DropMenu(window);
+        KIO::DropMenu *menu = new KIO::DropMenu();
         QObject::connect(menu, &QMenu::aboutToHide, menu, &QObject::deleteLater);
 
         // If the user clicks outside the menu, it will be destroyed without emitting the triggered signal.
@@ -530,6 +581,12 @@ void DropJobPrivate::slotDropActionDetermined(int error)
         });
 
         if (!(m_dropjobFlags & KIO::ShowMenuManually)) {
+            if (QWindow *parent = transientParent()) {
+                if (menu->winId()) {
+                    menu->windowHandle()->setTransientParent(parent);
+                }
+            }
+            auto *window = KJobWidgets::window(q);
             menu->popup(window ? window->mapToGlobal(m_relativePos) : QCursor::pos());
         }
         m_menus.insert(menu);
